@@ -1,7 +1,24 @@
 /* vim: set ts=4 sts=4 sw=4 expandtab textwidth=112: */
+/*
+ * This is free software; you can redistribute it and/or modify it under
+ * the terms of the GNU Library General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU Library General Public
+ * License along with this library. If not, see <http://www.gnu.org/licenses/>.
+ */
+
+#define _POSIX_SOURCE /* feature test macro for fileno */
+#define _XOPEN_SOURCE /* feature test macro for fsync */
 
 #include <tilda-config.h>
-#include <debug.h>
+#include "debug.h"
 
 #include <confuse.h>
 #include <glib.h>
@@ -10,7 +27,7 @@
 #include <stdlib.h> /* atoi */
 #include <unistd.h> /* fsync */
 
-#include <configsys.h>
+#include "configsys.h"
 
 static cfg_t *tc;
 
@@ -73,9 +90,13 @@ static cfg_opt_t config_opts[] = {
     CFG_INT("auto_hide_time", 2000, CFGF_NONE),
     CFG_INT("on_last_terminal_exit", 0, CFGF_NONE),
     CFG_INT("palette_scheme", 0, CFGF_NONE),
+    CFG_INT("non_focus_pull_up_behaviour", 0, CFGF_NONE),
+
     /* The default monitor number is 0 */
     CFG_INT("show_on_monitor_number", 0, CFGF_NONE),
- 
+    /* The length of a tab title */
+    CFG_INT("title_max_length", 25, CFGF_NONE),
+
     /* int list */
     CFG_INT_LIST("palette", "{\
         0x2e2e, 0x3434, 0x3636,\
@@ -106,6 +127,7 @@ static cfg_opt_t config_opts[] = {
     CFG_INT("text_blue", 0xffff, CFGF_NONE),
 
     /* booleans */
+    CFG_BOOL("scroll_history_infinite", FALSE, CFGF_NONE),
     CFG_BOOL("scroll_background", TRUE, CFGF_NONE),
     CFG_BOOL("scroll_on_output", FALSE, CFGF_NONE),
     CFG_BOOL("notebook_border", FALSE, CFGF_NONE),
@@ -129,15 +151,19 @@ static cfg_opt_t config_opts[] = {
     CFG_BOOL("double_buffer", FALSE, CFGF_NONE),
     CFG_BOOL("auto_hide_on_focus_lost", FALSE, CFGF_NONE),
     CFG_BOOL("auto_hide_on_mouse_leave", FALSE, CFGF_NONE),
+    /* Whether we limit the length of a tab title */
+    CFG_BOOL("title_max_length_flag", TRUE, CFGF_NONE),
+    /* Whether to set a new tab's working dir to the current tab's */
+    CFG_BOOL("inherit_working_dir", TRUE, CFGF_NONE),
     CFG_END()
 };
 
 /* Define these here, so that we can enable a non-threadsafe version
  * without changing the code below. */
 #ifndef NO_THREADSAFE
-	static GStaticMutex mutex = G_STATIC_MUTEX_INIT;
-	#define config_mutex_lock() g_static_mutex_lock (&mutex)
-	#define config_mutex_unlock() g_static_mutex_unlock (&mutex)
+	static GMutex mutex;
+	#define config_mutex_lock() g_mutex_lock (&mutex)
+	#define config_mutex_unlock() g_mutex_unlock (&mutex)
 #else
 	#define config_mutex_lock()
 	#define config_mutex_unlock()
@@ -147,7 +173,7 @@ static cfg_opt_t config_opts[] = {
 #define CONFIGS_SAME   0
 #define CONFIG1_NEWER  1
 
-static gboolean compare_config_versions (const gchar *config1, const gchar *config2);
+static gboolean compare_config_versions (const gchar *config1, const gchar *config2) G_GNUC_UNUSED;
 
 
 
@@ -172,16 +198,17 @@ gint config_init (const gchar *config_file)
 			DEBUG_ERROR ("Problem parsing config");
 			g_printerr (_("Problem when opening config file\n"));
 			return 1;
-		} else if (ret == CFG_PARSE_ERROR) {
-			DEBUG_ERROR ("Problem parsing config");
-			g_printerr (_("Problem while parsing config file\n"));
-        } else if (ret != CFG_SUCCESS) {
+		} else if (ret != CFG_SUCCESS) {
             DEBUG_ERROR ("Problem parsing config.");
 			g_printerr (_("An unexpected error occured while "
                 "parsing the config file\n"));
         }
 	}
 
+    #ifndef NO_THREADSAFE
+        g_mutex_init(&mutex);
+    #endif
+
 	return 0;
 }
 
@@ -218,6 +245,22 @@ gint config_setnint(const gchar *key, const gint val, const guint idx)
 	return 0;
 }
 
+gint config_setdouble (const gchar *key, const gdouble val) {
+	config_mutex_lock ();
+	cfg_setfloat (tc, key, val);
+	config_mutex_unlock ();
+
+	return 0;
+}
+
+gint config_setndouble (const gchar *key, const gdouble val, const guint idx) {
+	config_mutex_lock ();
+	cfg_setnfloat (tc, key, val, idx);
+	config_mutex_unlock ();
+
+	return 0;
+}
+
 gint config_setstr (const gchar *key, const gchar *val)
 {
 	config_mutex_lock ();
@@ -258,6 +301,26 @@ glong config_getnint(const gchar *key, const guint idx)
 	return temp;
 }
 
+gdouble config_getdouble (const gchar* key) {
+	gdouble temp;
+
+	config_mutex_lock ();
+	temp = cfg_getfloat (tc, key);
+	config_mutex_unlock ();
+
+	return temp;
+}
+
+gdouble config_getndouble (const gchar* key, const guint idx) {
+	gdouble temp;
+
+	config_mutex_lock ();
+	temp = cfg_getnfloat (tc, key, idx);
+	config_mutex_unlock ();
+
+	return temp;
+}
+
 gchar* config_getstr (const gchar *key)
 {
 	gchar *temp;
