@@ -10,27 +10,29 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU Library General Public
- * License along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * License along with this library. If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef TILDA_KEY_GRABBER_C
 #define TILDA_KEY_GRABBER_C
 
-#include <gtk/gtk.h>
-#include <confuse.h>
 #include "tilda_window.h"
 
-G_BEGIN_DECLS;
+G_BEGIN_DECLS
 
-#define PULL_TOGGLE 0
-#define PULL_DOWN 1
-#define PULL_UP 2
+enum pull_state { PULL_UP, PULL_DOWN, PULL_TOGGLE };
+void pull (struct tilda_window_ *tw, enum pull_state state, gboolean force_hide);
 
-void pull (tilda_window *tw);
-void *wait_for_signal (tilda_window *tw);
-void generate_animation_positions (tilda_window *tw);
+extern void generate_animation_positions (tilda_window *tw);
 
-G_END_DECLS;
+gboolean tilda_keygrabber_bind (const gchar *keystr, tilda_window *tw);
+void tilda_keygrabber_unbind (const gchar *keystr);
+
+/**
+ * This function will make the tilda window active after it starts
+ */
+void tilda_window_set_active (tilda_window *tw);
+
+G_END_DECLS
 
 #endif
