@@ -10,20 +10,18 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU Library General Public
- * License along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * License along with this library. If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <tilda-config.h>
 
-#include <debug.h>
-#include <tilda.h>
-#include <tilda_window.h>
-#include <tilda_terminal.h>
-#include <callback_func.h>
-#include <configsys.h>
-#include <wizard.h> /* wizard */
-#include <gtk/gtk.h>
+#include "debug.h"
+#include "tilda.h"
+#include "tilda_window.h"
+#include "tilda_terminal.h"
+#include "callback_func.h"
+#include "configsys.h"
+#include "wizard.h" /* wizard */
 
 #include <stdio.h>
 #include <stdlib.h> /* malloc */
@@ -36,15 +34,15 @@
 
 #define HTTP_REGEXP "(ftp|http)s?://[-a-zA-Z0-9.?$%&/=_~#.,:;+]*"
 
-GdkColor current_palette[TERMINAL_PALETTE_SIZE];
+GdkRGBA current_palette[TERMINAL_PALETTE_SIZE];
 
-static gint start_shell (struct tilda_term_ *tt, gboolean ignore_custom_command);
+static gint start_shell (struct tilda_term_ *tt, gboolean ignore_custom_command, const char* working_dir);
 static gint tilda_term_config_defaults (tilda_term *tt);
 static void child_exited_cb (GtkWidget *widget, gpointer data);
 static void window_title_changed_cb (GtkWidget *widget, gpointer data);
 static void status_line_changed_cb (GtkWidget *widget, gpointer data);
 static int button_press_cb (GtkWidget *widget, GdkEventButton *event, gpointer data);
-static void status_line_changed_cb (GtkWidget *widget, gpointer data);
+static gboolean key_press_cb (GtkWidget *widget, GdkEvent  *event, tilda_term *terminal);
 static void iconify_window_cb (GtkWidget *widget, gpointer data);
 static void deiconify_window_cb (GtkWidget *widget, gpointer data);
 static void raise_window_cb (GtkWidget *widget, gpointer data);
@@ -77,6 +75,9 @@ struct tilda_term_ *tilda_term_init (struct tilda_window_ *tw)
     int ret;
     struct tilda_term_ *term;
     GError *error = NULL;
+    tilda_term *current_tt;
+    gint current_tt_index;
+    char *current_tt_dir = NULL;
 
     term = g_malloc (sizeof (struct tilda_term_));
 
@@ -84,6 +85,9 @@ struct tilda_term_ *tilda_term_init (struct tilda_window_ *tw)
     if (!term)
         return NULL;
 
+    /* Set the PID to unset value */
+    term->pid = -1;
+
     /* Add the parent window reference */
     term->tw = tw;
 
@@ -97,6 +101,9 @@ struct tilda_term_ *tilda_term_init (struct tilda_window_ *tw)
     term->scrollbar = gtk_scrollbar_new (GTK_ORIENTATION_VERTICAL,
         gtk_scrollable_get_vadjustment (GTK_SCROLLABLE (VTE_TERMINAL(term->vte_term))));
 
+    /* Initialize to false, we have not yet dropped to the default shell */
+    term->dropped_to_default_shell = FALSE;
+
     /* Set properties of the terminal */
     tilda_term_config_defaults (term);
 
@@ -119,6 +126,8 @@ struct tilda_term_ *tilda_term_init (struct tilda_window_ *tw)
                       G_CALLBACK(status_line_changed_cb), term);
     g_signal_connect (G_OBJECT(term->vte_term), "button-press-event",
                       G_CALLBACK(button_press_cb), term);
+    g_signal_connect (G_OBJECT(term->vte_term), "key-press-event",
+		      G_CALLBACK(key_press_cb), term); //needs GDK_KEY_PRESS_MASK
 
     /* Connect to application request signals. */
     g_signal_connect (G_OBJECT(term->vte_term), "iconify-window",
@@ -154,8 +163,18 @@ struct tilda_term_ *tilda_term_init (struct tilda_window_ *tw)
     gtk_widget_show (term->vte_term);
     gtk_widget_show (term->hbox);
 
+    /* Get current term's working directory */
+    current_tt_index = gtk_notebook_get_current_page (GTK_NOTEBOOK(tw->notebook));
+    current_tt = g_list_nth_data (tw->terms, current_tt_index);
+    if (current_tt != NULL)
+    {
+        current_tt_dir = tilda_term_get_cwd(current_tt);
+    }
+
     /* Fork the appropriate command into the terminal */
-    ret = start_shell (term, FALSE);
+    ret = start_shell (term, FALSE, current_tt_dir);
+
+    g_free(current_tt_dir);
 
     if (ret)
         goto err_fork;
@@ -173,22 +192,13 @@ void tilda_term_set_scrollbar_position (tilda_term *tt, enum tilda_term_scrollba
     DEBUG_ASSERT (tt != NULL);
     DEBUG_ASSERT (pos == LEFT || pos == RIGHT || pos == DISABLED);
 
-    switch (pos)
-    {
-        case LEFT:
-            gtk_box_reorder_child (GTK_BOX(tt->hbox), tt->scrollbar, 0);
-            gtk_widget_show (tt->scrollbar);
-            break;
-
-        case RIGHT:
-            gtk_box_reorder_child (GTK_BOX(tt->hbox), tt->scrollbar, 1);
-            gtk_widget_show (tt->scrollbar);
-            break;
-
-        case DISABLED:
-        default:
-            gtk_widget_hide (tt->scrollbar);
-            break;
+    if (pos == DISABLED) {
+        gtk_widget_hide (tt->scrollbar);
+    } else {
+        /* We have already asserted that it's either disabled (already taken care of),
+         * left, or right, so no need to check twice. */
+        gtk_box_reorder_child (GTK_BOX(tt->hbox), tt->scrollbar, (pos == LEFT) ? 0 : 1);
+        gtk_widget_show (tt->scrollbar);
     }
 }
 
@@ -203,12 +213,22 @@ static void window_title_changed_cb (GtkWidget *widget, gpointer data)
     GtkWidget *label;
 
     label = gtk_notebook_get_tab_label (GTK_NOTEBOOK (tt->tw->notebook), tt->hbox);
-    gtk_label_set_text (GTK_LABEL(label), title);
+
+    guint length = config_getint ("title_max_length");
+
+    if(config_getbool("title_max_length_flag") && strlen(title) > length) {
+        gchar *titleOffset = title + strlen(title) - length;
+        gchar *shortTitle = g_strdup_printf ("...%s", titleOffset);
+        gtk_label_set_text (GTK_LABEL(label), shortTitle);
+        g_free(shortTitle);
+    } else {
+        gtk_label_set_text (GTK_LABEL(label), title);
+    }
 
     g_free (title);
 }
 
-static void status_line_changed_cb (GtkWidget *widget, gpointer data)
+static void status_line_changed_cb (GtkWidget *widget, G_GNUC_UNUSED gpointer data)
 {
     DEBUG_FUNCTION ("status_line_changed_cb");
     DEBUG_ASSERT (widget != NULL);
@@ -216,7 +236,7 @@ static void status_line_changed_cb (GtkWidget *widget, gpointer data)
     g_print ("Status = `%s'.\n", vte_terminal_get_status_line (VTE_TERMINAL(widget)));
 }
 
-static void iconify_window_cb (GtkWidget *widget, gpointer data)
+static void iconify_window_cb (G_GNUC_UNUSED GtkWidget *widget, gpointer data)
 {
     DEBUG_FUNCTION ("iconify_window_cb");
     DEBUG_ASSERT (data != NULL);
@@ -226,7 +246,7 @@ static void iconify_window_cb (GtkWidget *widget, gpointer data)
             gdk_window_iconify (gtk_widget_get_window ((GTK_WIDGET (data))));
 }
 
-static void deiconify_window_cb (GtkWidget *widget, gpointer data)
+static void deiconify_window_cb (G_GNUC_UNUSED GtkWidget *widget, gpointer data)
 {
     DEBUG_FUNCTION ("deiconify_window_cb");
     DEBUG_ASSERT (data != NULL);
@@ -236,7 +256,7 @@ static void deiconify_window_cb (GtkWidget *widget, gpointer data)
             gdk_window_deiconify (gtk_widget_get_window ((GTK_WIDGET (data))));
 }
 
-static void raise_window_cb (GtkWidget *widget, gpointer data)
+static void raise_window_cb (G_GNUC_UNUSED GtkWidget *widget, gpointer data)
 {
     DEBUG_FUNCTION ("raise_window_cb");
     DEBUG_ASSERT (data != NULL);
@@ -246,7 +266,7 @@ static void raise_window_cb (GtkWidget *widget, gpointer data)
             gdk_window_raise (gtk_widget_get_window (GTK_WIDGET (data)));
 }
 
-static void lower_window_cb (GtkWidget *widget, gpointer data)
+static void lower_window_cb (G_GNUC_UNUSED GtkWidget *widget, gpointer data)
 {
     DEBUG_FUNCTION ("lower_window_cb");
     DEBUG_ASSERT (data != NULL);
@@ -256,7 +276,7 @@ static void lower_window_cb (GtkWidget *widget, gpointer data)
             gdk_window_lower (gtk_widget_get_window (GTK_WIDGET (data)));
 }
 
-static void maximize_window_cb (GtkWidget *widget, gpointer data)
+static void maximize_window_cb (G_GNUC_UNUSED GtkWidget *widget, gpointer data)
 {
     DEBUG_FUNCTION ("maximize_window_cb");
     DEBUG_ASSERT (data != NULL);
@@ -266,7 +286,7 @@ static void maximize_window_cb (GtkWidget *widget, gpointer data)
             gdk_window_maximize (gtk_widget_get_window (GTK_WIDGET(data)));
 }
 
-static void restore_window_cb (GtkWidget *widget, gpointer data)
+static void restore_window_cb (G_GNUC_UNUSED GtkWidget *widget, gpointer data)
 {
     DEBUG_FUNCTION ("restore_window_cb");
     DEBUG_ASSERT (data != NULL);
@@ -276,7 +296,7 @@ static void restore_window_cb (GtkWidget *widget, gpointer data)
             gdk_window_unmaximize (gtk_widget_get_window (GTK_WIDGET (data)));
 }
 
-static void refresh_window_cb (GtkWidget *widget, gpointer data)
+static void refresh_window_cb (G_GNUC_UNUSED GtkWidget *widget, gpointer data)
 {
     DEBUG_FUNCTION ("refresh_window_cb");
     DEBUG_ASSERT (data != NULL);
@@ -296,7 +316,7 @@ static void refresh_window_cb (GtkWidget *widget, gpointer data)
     }
 }
 
-static void move_window_cb (GtkWidget *widget, guint x, guint y, gpointer data)
+static void move_window_cb (G_GNUC_UNUSED GtkWidget *widgets, guint x, guint y, gpointer data)
 {
     DEBUG_FUNCTION ("move_window_cb");
     DEBUG_ASSERT (data != NULL);
@@ -361,6 +381,37 @@ static void decrease_font_size_cb (GtkWidget *widget, gpointer data)
     adjust_font_size (widget, data, -1);
 }
 
+/* Returns the working directory of the terminal
+ *
+ * @param tt the tilda_term to get working directory of
+ *
+ * SUCCESS: return non-NULL char* that should be freed with g_free when done
+ * FAILURE: return NULL
+*/
+char* tilda_term_get_cwd(struct tilda_term_* tt)
+{
+    char *file;
+    char *cwd;
+    GError *error = NULL;
+
+    if (tt->pid < 0)
+    {
+        return NULL;
+    }
+
+    file = g_strdup_printf ("/proc/%d/cwd", tt->pid);
+    cwd = g_file_read_link (file, &error);
+    g_free (file);
+
+    if (cwd == NULL)
+    {
+        g_printerr (_("Problem reading link %s: %s\n"), file, error->message);
+        g_error_free (error);
+    }
+
+    return cwd;
+}
+
 /* Fork a shell into the VTE Terminal
  *
  * @param tt the tilda_term to fork into
@@ -368,7 +419,7 @@ static void decrease_font_size_cb (GtkWidget *widget, gpointer data)
  * SUCCESS: return 0
  * FAILURE: return non-zero
  */
-static gint start_shell (struct tilda_term_ *tt, gboolean ignore_custom_command)
+static gint start_shell (struct tilda_term_ *tt, gboolean ignore_custom_command, const char* working_dir)
 {
     DEBUG_FUNCTION ("start_shell");
     DEBUG_ASSERT (tt != NULL);
@@ -380,6 +431,11 @@ static gint start_shell (struct tilda_term_ *tt, gboolean ignore_custom_command)
 
     gchar *default_command;
 
+    if (working_dir == NULL || config_getbool ("inherit_working_dir") == FALSE)
+    {
+        working_dir = config_getstr ("working_dir");
+    }
+
     if (config_getbool ("run_command") && !ignore_custom_command)
     {
         ret = g_shell_parse_argv (config_getstr ("command"), &argc, &argv, &error);
@@ -400,13 +456,13 @@ static gint start_shell (struct tilda_term_ *tt, gboolean ignore_custom_command)
 
         ret = vte_terminal_fork_command_full (VTE_TERMINAL (tt->vte_term),
             VTE_PTY_DEFAULT, /* VtePtyFlags pty_flags */
-            config_getstr ("working_dir"), /* const char *working_directory */
+            working_dir, /* const char *working_directory */
             argv, /* char **argv */
             envv, /* char **envv */
             G_SPAWN_SEARCH_PATH,    /* GSpawnFlags spawn_flags */
             NULL, /* GSpawnChildSetupFunc child_setup */
             NULL, /* gpointer child_setup_data */
-            NULL, /* GPid *child_pid */
+            &tt->pid, /* GPid *child_pid */
             NULL  /* GError **error */
             );
 
@@ -427,6 +483,20 @@ static gint start_shell (struct tilda_term_ *tt, gboolean ignore_custom_command)
 
 launch_default_shell:
 
+    /* If we have dropped to the default shell before, then this time, we
+     * do not spawn a new shell, but instead close the current shell. This will
+     * cause the current tab to close.
+     */
+    if (tt->dropped_to_default_shell) {
+        gint index = gtk_notebook_page_num (GTK_NOTEBOOK(tt->tw->notebook),
+            tt->hbox);
+        tilda_window_close_tab (tt->tw, index, FALSE);
+        return 0;
+    }
+    if (ignore_custom_command) {
+        tt->dropped_to_default_shell = TRUE;
+    }
+
     /* No custom command, get it from the environment */
     default_command = (gchar *) g_getenv ("SHELL");
 
@@ -443,13 +513,13 @@ launch_default_shell:
 
     ret = vte_terminal_fork_command_full (VTE_TERMINAL (tt->vte_term),
         VTE_PTY_DEFAULT, /* VtePtyFlags pty_flags */
-        config_getstr ("working_dir"), /* const char *working_directory */
+        working_dir, /* const char *working_directory */
         argv, /* char **argv */
         NULL, /* char **envv */
         0,    /* GSpawnFlags spawn_flags */
         NULL, /* GSpawnChildSetupFunc child_setup */
         NULL, /* gpointer child_setup_data */
-        NULL, /* GPid *child_pid */
+        &tt->pid, /* GPid *child_pid */
         NULL  /* GError **error */
         );
 
@@ -494,10 +564,10 @@ static void child_exited_cb (GtkWidget *widget, gpointer data)
             break;
         case RESTART_COMMAND:
             vte_terminal_feed (VTE_TERMINAL(tt->vte_term), "\r\n\r\n", 4);
-            start_shell (tt, FALSE);
+            start_shell (tt, FALSE, NULL);
             break;
         case DROP_TO_DEFAULT_SHELL:
-            start_shell (tt, TRUE);
+            start_shell (tt, TRUE, NULL);
         default:
             break;
     }
@@ -517,28 +587,29 @@ static gint tilda_term_config_defaults (tilda_term *tt)
     DEBUG_ASSERT (tt != NULL);
 
     gdouble transparency_level = 0.0;
-    GdkColor fg, bg /*, tint, highlight, cursor, black */;
+    GdkRGBA fg, bg;
     gchar* word_chars;
     gint i;
 
     /** Colors & Palette **/
-    bg.red   =    config_getint ("back_red");
-    bg.green =    config_getint ("back_green");
-    bg.blue  =    config_getint ("back_blue");
+    bg.red   =    GUINT16_TO_FLOAT(config_getint ("back_red"));
+    bg.green =    GUINT16_TO_FLOAT(config_getint ("back_green"));
+    bg.blue  =    GUINT16_TO_FLOAT(config_getint ("back_blue"));
+    bg.alpha =    1.0;
 
-    fg.red   =    config_getint ("text_red");
-    fg.green =    config_getint ("text_green");
-    fg.blue  =    config_getint ("text_blue");
+    fg.red   =    GUINT16_TO_FLOAT(config_getint ("text_red"));
+    fg.green =    GUINT16_TO_FLOAT(config_getint ("text_green"));
+    fg.blue  =    GUINT16_TO_FLOAT(config_getint ("text_blue"));
+    fg.alpha =    1.0;
 
-    for(i = 0;i < TERMINAL_PALETTE_SIZE; i++)
-    {
-        current_palette[i].pixel = 0;
-        current_palette[i].red   = config_getnint ("palette", i*3);
-        current_palette[i].green = config_getnint ("palette", i*3+1);
-        current_palette[i].blue  = config_getnint ("palette", i*3+2);
+    for(i = 0;i < TERMINAL_PALETTE_SIZE; i++) {
+        current_palette[i].red   = GUINT16_TO_FLOAT(config_getnint ("palette", i*3));
+        current_palette[i].green = GUINT16_TO_FLOAT(config_getnint ("palette", i*3+1));
+        current_palette[i].blue  = GUINT16_TO_FLOAT(config_getnint ("palette", i*3+2));
+        current_palette[i].alpha = 1.0;
     }
 
-    vte_terminal_set_colors (VTE_TERMINAL(tt->vte_term), &fg, &bg, current_palette, TERMINAL_PALETTE_SIZE);
+    vte_terminal_set_colors_rgba (VTE_TERMINAL(tt->vte_term), &fg, &bg, current_palette, TERMINAL_PALETTE_SIZE);
 
     /** Bells **/
     vte_terminal_set_audible_bell (VTE_TERMINAL(tt->vte_term), config_getbool ("bell"));
@@ -662,7 +733,7 @@ menu_preferences_cb (GtkWidget *widget, gpointer data)
 }
 
 static void
-menu_quit_cb (GtkWidget *widget, gpointer data)
+menu_quit_cb (G_GNUC_UNUSED GtkWidget *widget, G_GNUC_UNUSED gpointer data)
 {
     DEBUG_FUNCTION ("menu_quit_cb");
 
@@ -797,7 +868,7 @@ static void popup_menu (tilda_window *tw, tilda_term *tt)
     gtk_widget_show_all(menu);
 }
 
-static int button_press_cb (GtkWidget *widget, GdkEventButton *event, gpointer data)
+static int button_press_cb (G_GNUC_UNUSED GtkWidget *widget, GdkEventButton *event, gpointer data)
 {
     DEBUG_FUNCTION ("button_press_cb");
     DEBUG_ASSERT (data != NULL);
@@ -806,7 +877,7 @@ static int button_press_cb (GtkWidget *widget, GdkEventButton *event, gpointer d
     tilda_term *tt;
     gchar *match;
     gint tag;
-    gint xpad, ypad;
+    gint ypad;
     gchar *cmd;
     gchar *web_browser_cmd;
     gboolean ret = FALSE;
@@ -815,6 +886,12 @@ static int button_press_cb (GtkWidget *widget, GdkEventButton *event, gpointer d
 
     switch (event->button)
     {
+        case 9:
+            tilda_window_next_tab (tt->tw);
+            break;
+        case 8:
+            tilda_window_prev_tab (tt->tw);
+            break;
         case 3: /* Right Click */
             popup_menu (tt->tw, tt);
             break;
@@ -825,7 +902,7 @@ static int button_press_cb (GtkWidget *widget, GdkEventButton *event, gpointer d
             GtkBorder border;
             gtk_widget_style_get (GTK_WIDGET (terminal),
                 "inner-border", &border, NULL);
-            xpad = border.left;
+
             ypad = border.bottom;
             match = vte_terminal_match_check (terminal,
                     (event->x - ypad) /
@@ -838,7 +915,7 @@ static int button_press_cb (GtkWidget *widget, GdkEventButton *event, gpointer d
             if (match != NULL)
             {
 #if DEBUG
-                g_print ("Got a Ctrl+Left Click -- Matched: `%s' (%d)\n", match, tag);
+                g_print ("Got a Left Click -- Matched: `%s' (%d)\n", match, tag);
 #endif
                 web_browser_cmd = g_strescape (config_getstr ("web_browser"), NULL);
                 cmd = g_strdup_printf ("%s %s", web_browser_cmd, match);
@@ -869,4 +946,17 @@ static int button_press_cb (GtkWidget *widget, GdkEventButton *event, gpointer d
     return FALSE;
 }
 
+gboolean key_press_cb (GtkWidget *widget,
+                       GdkEvent  *event,
+                       tilda_term *terminal)
+{
+    if(event->type == GDK_KEY_PRESS) {
+        GdkEventKey *keyevent = (GdkEventKey*) event;
+        if(keyevent->keyval == GDK_KEY_Menu) {
+            popup_menu(terminal->tw, terminal);
+        }
+    }
+    return GDK_EVENT_PROPAGATE;
+}
+
 /* vim: set ts=4 sts=4 sw=4 expandtab: */
