@@ -1,5 +1,3 @@
-
-
 /*
  * This is free software; you can redistribute it and/or modify it under
  * the terms of the GNU Library General Public License as published by
@@ -12,19 +10,18 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU Library General Public
- * License along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * License along with this library. If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <tilda-config.h>
 
-#include <debug.h>
-#include <tilda.h>
-#include <callback_func.h>
-#include <configsys.h>
-#include <tilda_window.h>
-#include <tilda_terminal.h>
-#include <key_grabber.h>
+#include "debug.h"
+#include "tilda.h"
+#include "callback_func.h"
+#include "configsys.h"
+#include "tilda_window.h"
+#include "tilda_terminal.h"
+#include "key_grabber.h"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -35,6 +32,8 @@
 #include <glib.h>
 #include <glib/gi18n.h>
 #include <vte/vte.h>
+#include <X11/Xlib.h>
+#include <gdk/gdkx.h>
 
 static void
 tilda_window_setup_alpha_mode (tilda_window *tw)
@@ -96,22 +95,20 @@ void tilda_window_close_current_tab (tilda_window *tw)
 
 gint tilda_window_set_tab_position (tilda_window *tw, enum notebook_tab_positions pos)
 {
-    switch (pos)
-    {
-        default: /* default is top */
-            g_printerr (_("You have a bad tab_pos in your configuration file\n"));
-        case NB_TOP:
-            gtk_notebook_set_tab_pos (GTK_NOTEBOOK (tw->notebook), GTK_POS_TOP);
-            break;
-        case NB_BOTTOM:
-            gtk_notebook_set_tab_pos (GTK_NOTEBOOK (tw->notebook), GTK_POS_BOTTOM);
-            break;
-        case NB_LEFT:
-            gtk_notebook_set_tab_pos (GTK_NOTEBOOK (tw->notebook), GTK_POS_LEFT);
-            break;
-        case NB_RIGHT:
-            gtk_notebook_set_tab_pos (GTK_NOTEBOOK (tw->notebook), GTK_POS_RIGHT);
-            break;
+    const GtkPositionType gtk_pos[] = {GTK_POS_TOP, GTK_POS_BOTTOM, GTK_POS_LEFT, GTK_POS_RIGHT };
+
+    if ((pos < 0) || (pos > 4)) {
+        g_printerr (_("You have a bad tab_pos in your configuration file\n"));
+        pos = NB_TOP;
+    }
+
+    if(NB_HIDDEN == pos) {
+        gtk_notebook_set_show_tabs (GTK_NOTEBOOK(tw->notebook), FALSE);
+    }
+    else {
+        gtk_notebook_set_tab_pos (GTK_NOTEBOOK (tw->notebook), gtk_pos[pos]);
+        if (gtk_notebook_get_n_pages (GTK_NOTEBOOK (tw->notebook)) > 1)
+            gtk_notebook_set_show_tabs (GTK_NOTEBOOK(tw->notebook), TRUE);
     }
 
     return 0;
@@ -122,19 +119,26 @@ gint toggle_fullscreen_cb (tilda_window *tw)
 {
     DEBUG_FUNCTION ("toggle_fullscreen_cb");
     DEBUG_ASSERT (tw != NULL);
-    
-	if (tw->fullscreen != TRUE) {
-		tw->fullscreen = TRUE;
-		gtk_window_fullscreen (GTK_WINDOW (tw->window));
-	}
-	else {
-		gtk_window_unfullscreen (GTK_WINDOW (tw->window));
-		tw->fullscreen = FALSE;
-	}
+
+    if (tw->fullscreen != TRUE) {
+        gtk_window_fullscreen (GTK_WINDOW (tw->window));
+    }
+    else {
+        gtk_window_unfullscreen (GTK_WINDOW (tw->window));
+        // This appears to be necssary on (at least) xfwm4 if you tabbed out
+        // while fullscreened.
+        gtk_window_set_default_size (GTK_WINDOW(tw->window), config_getint ("max_width"), config_getint ("max_height"));
+        gtk_window_resize (GTK_WINDOW(tw->window), config_getint ("max_width"), config_getint ("max_height"));
+        gtk_window_move(GTK_WINDOW(tw->window), config_getint ("x_pos"), config_getint ("y_pos"));
+    }
+    tw->fullscreen = !tw->fullscreen;
+
+    // It worked. Having this return GDK_EVENT_STOP makes the callback not carry the
+    // keystroke into the vte terminal widget.
+    return GDK_EVENT_STOP;
 }
 
-
-static gint next_tab (tilda_window *tw)
+gint tilda_window_next_tab (tilda_window *tw)
 {
     DEBUG_FUNCTION ("next_tab");
     DEBUG_ASSERT (tw != NULL);
@@ -151,12 +155,12 @@ static gint next_tab (tilda_window *tw)
     else
       gtk_notebook_next_page (GTK_NOTEBOOK (tw->notebook));
 
-    // It worked. Having this return true makes the callback not carry the
+    // It worked. Having this return GDK_EVENT_STOP makes the callback not carry the
     // keystroke into the vte terminal widget.
-    return TRUE;
+    return GDK_EVENT_STOP;
 }
 
-static gint prev_tab (tilda_window *tw)
+gint tilda_window_prev_tab (tilda_window *tw)
 {
     DEBUG_FUNCTION ("prev_tab");
     DEBUG_ASSERT (tw != NULL);
@@ -172,14 +176,16 @@ static gint prev_tab (tilda_window *tw)
     else
       gtk_notebook_prev_page (GTK_NOTEBOOK (tw->notebook));
 
-    // It worked. Having this return true makes the callback not carry the
+    // It worked. Having this return GDK_EVENT_STOP makes the callback not carry the
     // keystroke into the vte terminal widget.
-    return TRUE;
+    return GDK_EVENT_STOP;
 }
 
-static gint move_tab_left (tilda_window *tw)
+enum tab_direction { TAB_LEFT = 1, TAB_RIGHT = -1 };
+
+static gint move_tab (tilda_window *tw, int direction)
 {
-    DEBUG_FUNCTION ("move_tab_left");
+    DEBUG_FUNCTION ("move_tab");
     DEBUG_ASSERT (tw != NULL);
 
     int num_pages;
@@ -188,55 +194,37 @@ static gint move_tab_left (tilda_window *tw)
     GtkWidget* current_page;
 
     num_pages = gtk_notebook_get_n_pages (GTK_NOTEBOOK (tw->notebook));
-    current_page_index = gtk_notebook_get_current_page (GTK_NOTEBOOK (tw->notebook));
-    current_page = gtk_notebook_get_nth_page (GTK_NOTEBOOK (tw->notebook),
-                                              current_page_index);
 
     if (num_pages > 1) {
-      if (current_page_index < num_pages - 1) {
-        // Move current page one to the left
-        new_page_index = current_page_index + 1;
-      } else {
-        // Current page is at beginning: move to end
-        new_page_index = 0;
-      }
+        current_page_index = gtk_notebook_get_current_page (GTK_NOTEBOOK (tw->notebook));
+        current_page = gtk_notebook_get_nth_page (GTK_NOTEBOOK (tw->notebook),
+                                                  current_page_index);
 
-      gtk_notebook_reorder_child (GTK_NOTEBOOK (tw->notebook), current_page,
-                                  new_page_index);
+        /* wrap over if new_page_index over-/underflows */
+        new_page_index = (current_page_index + direction) % num_pages;
+
+        gtk_notebook_reorder_child (GTK_NOTEBOOK (tw->notebook), current_page,
+                                    new_page_index);
     }
 
-    // It worked. Having this return true makes the callback not carry the
+    // It worked. Having this return GDK_EVENT_STOP makes the callback not carry the
     // keystroke into the vte terminal widget.
-    return TRUE;
+    return GDK_EVENT_STOP;
+
+}
+static gint move_tab_left (tilda_window *tw)
+{
+    DEBUG_FUNCTION ("move_tab_left");
+    return move_tab(tw, LEFT);
 }
 
 static gint move_tab_right (tilda_window *tw)
 {
     DEBUG_FUNCTION ("move_tab_right");
-    DEBUG_ASSERT (tw != NULL);
-
-    int num_pages;
-    int current_page_index;
-    GtkWidget* current_page;
-
-    num_pages = gtk_notebook_get_n_pages (GTK_NOTEBOOK (tw->notebook));
-    current_page_index = gtk_notebook_get_current_page (GTK_NOTEBOOK (tw->notebook));
-    current_page = gtk_notebook_get_nth_page (GTK_NOTEBOOK (tw->notebook),
-                                              current_page_index);
-
-    if (num_pages > 1) {
-      // Move current page one to the right. This automatically wraps
-      // when current_page_index - 1 is negative.
-      gtk_notebook_reorder_child (GTK_NOTEBOOK (tw->notebook), current_page,
-                                  current_page_index - 1);
-    }
-
-    // It worked. Having this return true makes the callback not carry the
-    // keystroke into the vte terminal widget.
-    return TRUE;
+    return move_tab(tw, RIGHT);
 }
 
-static void focus_term (GtkWidget *widget, gpointer data)
+static gboolean focus_term (GtkWidget *widget, gpointer data)
 {
     DEBUG_FUNCTION ("focus_term");
     DEBUG_ASSERT (data != NULL);
@@ -252,31 +240,37 @@ static void focus_term (GtkWidget *widget, gpointer data)
         list = gtk_container_get_children (GTK_CONTAINER(box));
         gtk_widget_grab_focus (list->data);
     }
+
+    // It worked. Having this return GDK_EVENT_STOP makes the callback not carry the
+    // keystroke into the vte terminal widget.
+    return GDK_EVENT_STOP;
 }
 
 static gboolean auto_hide_tick(gpointer data)
 {
     DEBUG_FUNCTION ("auto_hide_tick");
     DEBUG_ASSERT (data != NULL);
-    
+
     tilda_window *tw = TILDA_WINDOW(data);
     tw->auto_hide_current_time += tw->timer_resolution;
     if ((tw->auto_hide_current_time >= tw->auto_hide_max_time) || tw->current_state == UP)
     {
-        pull(tw, PULL_UP);
+        pull(tw, PULL_UP, TRUE);
         tw->auto_hide_tick_handler = 0;
         return FALSE;
     }
+
     return TRUE;
 }
 
 /* Start auto hide tick */
 static void start_auto_hide_tick(tilda_window *tw)
 {
+    DEBUG_FUNCTION("start_auto_hide_tick");
     // If the delay is less or equal to 1000ms then the timer is not precise
     // enough, because it takes already about 200ms to register it, so we
     // rather sleep for the given amount of time.
-    const int MAX_SLEEP_TIME = 1000;
+    const guint32 MAX_SLEEP_TIME = 1000;
 
     if ((tw->auto_hide_tick_handler == 0) && (tw->disable_auto_hide == FALSE)) {
         /* If there is not timer registered yet, then the auto_hide_tick_handler
@@ -287,13 +281,13 @@ static void start_auto_hide_tick(tilda_window *tw)
         if (tw->auto_hide_max_time > MAX_SLEEP_TIME) {
             tw->auto_hide_current_time = 0;
             tw->auto_hide_tick_handler = g_timeout_add(tw->timer_resolution, auto_hide_tick, tw);
-        } else if (tw->auto_hide_max_time <= MAX_SLEEP_TIME) {
+        } else if (tw->auto_hide_max_time > 0 && tw->auto_hide_max_time <= MAX_SLEEP_TIME) {
             // auto_hide_max_time is in milli seconds, so we need to convert to
             // microseconds by multiplying with 1000
             g_usleep (tw->auto_hide_max_time * 1000);
-            pull(tw, PULL_UP);
+            pull(tw, PULL_UP, TRUE);
         } else {
-            pull(tw, PULL_UP);
+            pull(tw, PULL_UP, TRUE);
         }
     }
 }
@@ -308,45 +302,70 @@ static void stop_auto_hide_tick(tilda_window *tw)
     }
 }
 
-static void mouse_enter (GtkWidget *widget, GdkEvent *event, gpointer data)
+static gboolean mouse_enter (GtkWidget *widget, G_GNUC_UNUSED GdkEvent *event, gpointer data)
 {
     DEBUG_FUNCTION ("mouse_enter");
     DEBUG_ASSERT (data != NULL);
     DEBUG_ASSERT (widget != NULL);
-    
+
     tilda_window *tw = TILDA_WINDOW(data);
     stop_auto_hide_tick(tw);
-    if (tw->disable_auto_hide == FALSE)
-        tilda_window_set_active(tw);
+
+    return GDK_EVENT_STOP;
 }
- 
-static void mouse_leave (GtkWidget *widget, GdkEvent *event, gpointer data)
+
+static gboolean mouse_leave (GtkWidget *widget, G_GNUC_UNUSED GdkEvent *event, gpointer data)
 {
     DEBUG_FUNCTION ("mouse_leave");
     DEBUG_ASSERT (data != NULL);
     DEBUG_ASSERT (widget != NULL);
-    
+
     GdkEventCrossing *ev = (GdkEventCrossing*)event;
     tilda_window *tw = TILDA_WINDOW(data);
-    
+
     if ((ev->mode != GDK_CROSSING_NORMAL) || (tw->auto_hide_on_mouse_leave == FALSE))
-        return;
-    
+        return GDK_EVENT_STOP;
+
     start_auto_hide_tick(tw);
+
+    return GDK_EVENT_STOP;
 }
 
-static void focus_out_event_cb (GtkWidget *widget, GdkEvent *event, gpointer data)
+static gboolean focus_out_event_cb (GtkWidget *widget, G_GNUC_UNUSED GdkEvent *event, gpointer data)
 {
     DEBUG_FUNCTION ("focus_out_event_cb");
     DEBUG_ASSERT (data != NULL);
     DEBUG_ASSERT (widget != NULL);
-    
+
     tilda_window *tw = TILDA_WINDOW(data);
-    
+
+    /**
+    * When the tilda 'key' to pull down/up the tilda window is pressed, then tilda will inevitably loose focus. The
+    * problem is that we cannot distinguish whether it was focused before the key press occurred. Checking the xevent
+    * type here allows us to distinguish these two cases:
+    *
+    *  * We loose focus because of a KeyPress event
+    *  * We loose focus because another window gained focus or some other reason.
+    *
+    *  Depending on one of the two cases we set the tw->focus_loss_on_keypress to true or false. We can then
+    *  check this flag in the pull() function that shows or hides the tilda window. This helps us to decide if
+    *  we just need to focus tilda or if we should hide it.
+    */
+    XEvent xevent;
+    XPeekEvent(gdk_x11_display_get_xdisplay(gdk_window_get_display(gtk_widget_get_window(tw->window))), &xevent);
+
+    if(xevent.type == KeyPress) {
+        tw->focus_loss_on_keypress = TRUE;
+    } else {
+        tw->focus_loss_on_keypress = FALSE;
+    }
+
     if (tw->auto_hide_on_focus_lost == FALSE)
-        return;
-    
+        return GDK_EVENT_PROPAGATE;
+
     start_auto_hide_tick(tw);
+
+    return GDK_EVENT_PROPAGATE;
 }
 
 static void goto_tab (tilda_window *tw, guint i)
@@ -357,7 +376,7 @@ static void goto_tab (tilda_window *tw, guint i)
     gtk_notebook_set_current_page (GTK_NOTEBOOK (tw->notebook), i);
 }
 
-static gboolean goto_tab_generic (tilda_window *tw, gint tab_number)
+static gboolean goto_tab_generic (tilda_window *tw, guint tab_number)
 {
     DEBUG_FUNCTION ("goto_tab_generic");
     DEBUG_ASSERT (tw != NULL);
@@ -365,10 +384,9 @@ static gboolean goto_tab_generic (tilda_window *tw, gint tab_number)
     if (g_list_length (tw->terms) > (tab_number-1))
     {
         goto_tab (tw, tab_number - 1);
-        return TRUE;
     }
 
-    return FALSE;
+    return TRUE;
 }
 
 /* These all just call the generic function since they're all basically the same
@@ -422,40 +440,40 @@ static gint cpaste (tilda_window *tw)
 }
 
 /* Tie a single keyboard shortcut to a callback function */
-gint tilda_add_config_accelerator(const gchar* key, GCallback callback_func, tilda_window *tw)
+static gint tilda_add_config_accelerator(const gchar* key, GCallback callback_func, tilda_window *tw)
 {
     guint accel_key;
     GdkModifierType accel_mods;
     GClosure *temp;
- 
+
     gtk_accelerator_parse (config_getstr(key), &accel_key, &accel_mods);
     if (! ((accel_key == 0) && (accel_mods == 0)) )  // make sure it parsed properly
     {
         temp = g_cclosure_new_swap (callback_func, tw, NULL);
         gtk_accel_group_connect (tw->accel_group, accel_key, accel_mods , GTK_ACCEL_VISIBLE, temp);
     }
- 
+
     return 0;
 }
- 
+
 gint tilda_window_setup_keyboard_accelerators (tilda_window *tw)
 {
- 
+
     /* If we already have an tw->accel_group (which would happen if we're redefining accelerators in the config window)
        we want to remove it before creating a new one. */
     if (tw->accel_group != NULL)
         gtk_window_remove_accel_group (GTK_WINDOW (tw->window), tw->accel_group);
- 
+
     /* Create Accel Group to add key codes for quit, next, prev and new tabs */
     tw->accel_group = gtk_accel_group_new ();
     gtk_window_add_accel_group (GTK_WINDOW (tw->window), tw->accel_group);
- 
+
     /* Set up keyboard shortcuts for Exit, Next Tab, Previous Tab,
        Move Tab, Add Tab, Close Tab, Copy, and Paste using key
        combinations defined in the config. */
     tilda_add_config_accelerator("quit_key",         G_CALLBACK(gtk_main_quit),                  tw);
-    tilda_add_config_accelerator("nexttab_key",      G_CALLBACK(next_tab),                       tw);
-    tilda_add_config_accelerator("prevtab_key",      G_CALLBACK(prev_tab),                       tw);
+    tilda_add_config_accelerator("nexttab_key",      G_CALLBACK(tilda_window_next_tab),          tw);
+    tilda_add_config_accelerator("prevtab_key",      G_CALLBACK(tilda_window_prev_tab),          tw);
     tilda_add_config_accelerator("movetableft_key",  G_CALLBACK(move_tab_left),                  tw);
     tilda_add_config_accelerator("movetabright_key", G_CALLBACK(move_tab_right),                 tw);
     tilda_add_config_accelerator("addtab_key",       G_CALLBACK(tilda_window_add_tab),           tw);
@@ -463,7 +481,7 @@ gint tilda_window_setup_keyboard_accelerators (tilda_window *tw)
     tilda_add_config_accelerator("copy_key",         G_CALLBACK(ccopy),                          tw);
     tilda_add_config_accelerator("paste_key",        G_CALLBACK(cpaste),                         tw);
     tilda_add_config_accelerator("fullscreen_key",   G_CALLBACK(toggle_fullscreen_cb),           tw);
- 
+
     /* Set up keyboard shortcuts for Goto Tab # using key combinations defined in the config*/
     /* Know a better way? Then you do. */
     tilda_add_config_accelerator("gototab_1_key",  G_CALLBACK(goto_tab_1),  tw);
@@ -476,7 +494,7 @@ gint tilda_window_setup_keyboard_accelerators (tilda_window *tw)
     tilda_add_config_accelerator("gototab_8_key",  G_CALLBACK(goto_tab_8),  tw);
     tilda_add_config_accelerator("gototab_9_key",  G_CALLBACK(goto_tab_9),  tw);
     tilda_add_config_accelerator("gototab_10_key", G_CALLBACK(goto_tab_10), tw);
-    
+
     return 0;
 }
 
@@ -498,40 +516,22 @@ static gint tilda_window_set_icon (tilda_window *tw, gchar *filename)
     return 0;
 }
 
-static gboolean delete_event_callback (GtkWidget *widget,
-                                GdkEvent  *event,
-                                gpointer   user_data)
+static gboolean delete_event_callback (G_GNUC_UNUSED GtkWidget *widget,
+                                G_GNUC_UNUSED GdkEvent  *event,
+                                G_GNUC_UNUSED gpointer   user_data)
 {
     gtk_main_quit ();
+    return FALSE;
 }
 
-/**
- * tilda_window_init ()
- *
- * Create a new tilda_window * and return it. It will also initialize and set up
- * as much of the window as possible using the values in the configuation system.
- *
- * @param instance the instance number of this tilda_window
- *
- * Success: return a non-NULL tilda_window *
- * Failure: return NULL
- *
- * Notes: The configuration system must be up and running before calling this function.
- */
-tilda_window *tilda_window_init (const gchar *config_file, const gint instance)
+gboolean tilda_window_init (const gchar *config_file, const gint instance, tilda_window *tw)
 {
     DEBUG_FUNCTION ("tilda_window_init");
     DEBUG_ASSERT (instance >= 0);
 
-    tilda_window *tw;
     GtkCssProvider *provider;
     GtkStyleContext *style_context;
 
-    tw = g_malloc (sizeof(tilda_window));
-
-    if (tw == NULL)
-        return NULL;
-
     /* Set the instance number */
     tw->instance = instance;
 
@@ -540,18 +540,26 @@ tilda_window *tilda_window_init (const gchar *config_file, const gint instance)
 
     /* Create the main window */
     tw->window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
-    
+
     /* Generic timer resolution */
     tw->timer_resolution = config_getint("timer_resolution");
-    
+
     /* Auto hide support */
     tw->auto_hide_tick_handler = 0;
     tw->auto_hide_max_time = config_getint("auto_hide_time");
     tw->auto_hide_on_mouse_leave = config_getbool("auto_hide_on_mouse_leave");
     tw->auto_hide_on_focus_lost = config_getbool("auto_hide_on_focus_lost");
     tw->disable_auto_hide = FALSE;
+    tw->focus_loss_on_keypress = FALSE;
 
-	tw->fullscreen = FALSE;
+    if(1 == config_getint("non_focus_pull_up_behaviour")) {
+        tw->hide_non_focused = TRUE;
+    }
+    else {
+        tw->hide_non_focused = FALSE;
+    }
+
+    tw->fullscreen = FALSE;
 
     /* Set up all window properties */
     if (config_getbool ("pinned"))
@@ -614,13 +622,19 @@ tilda_window *tilda_window_init (const gchar *config_file, const gint instance)
     /* Add the initial terminal */
     if (!tilda_window_add_tab (tw))
     {
-        free (tw);
-        return NULL;
+        return FALSE;
     }
 
+    /* This is required in key_grabber.c to get the x11 server time,
+     * since the specification requires this flag to be set when
+     * gdk_x11_get_server_time() is called.
+     **/
+    gtk_widget_add_events (tw->window, GDK_PROPERTY_CHANGE_MASK );
+
     /* Connect signal handlers */
     g_signal_connect (G_OBJECT(tw->window), "delete-event", G_CALLBACK (delete_event_callback), tw);
     g_signal_connect (G_OBJECT(tw->window), "show", G_CALLBACK (focus_term), tw);
+
     g_signal_connect (G_OBJECT(tw->window), "focus-out-event", G_CALLBACK (focus_out_event_cb), tw);
     g_signal_connect (G_OBJECT(tw->window), "enter-notify-event", G_CALLBACK (mouse_enter), tw);
     g_signal_connect (G_OBJECT(tw->window), "leave-notify-event", G_CALLBACK (mouse_leave), tw);
@@ -637,11 +651,11 @@ tilda_window *tilda_window_init (const gchar *config_file, const gint instance)
     gtk_window_set_default_size (GTK_WINDOW(tw->window), config_getint ("max_width"), config_getint ("max_height"));
     gtk_window_resize (GTK_WINDOW(tw->window), config_getint ("max_width"), config_getint ("max_height"));
 
-	/* Create GDK resources now, to prevent crashes later on */
+    /* Create GDK resources now, to prevent crashes later on */
     gtk_widget_realize (tw->window);
     generate_animation_positions (tw);
 
-    return tw;
+    return TRUE;
 }
 
 gint tilda_window_free (tilda_window *tw)
@@ -660,19 +674,10 @@ gint tilda_window_free (tilda_window *tw)
     }
 
     g_free (tw->config_file);
-    g_free (tw);
 
     return 0;
 }
 
-/**
- * tilda_window_add_tab ()
- *
- * Create and add a new tab at the end of the notebook
- *
- * Success: the new tab's index (>=0)
- * Failure: -1
- */
 gint tilda_window_add_tab (tilda_window *tw)
 {
     DEBUG_FUNCTION ("tilda_window_add_tab");
@@ -696,10 +701,12 @@ gint tilda_window_add_tab (tilda_window *tw)
     /* Strangely enough, prepend puts pages on the end */
     index = gtk_notebook_append_page (GTK_NOTEBOOK(tw->notebook), tt->hbox, label);
     gtk_notebook_set_current_page (GTK_NOTEBOOK(tw->notebook), index);
-	gtk_notebook_set_tab_reorderable(GTK_NOTEBOOK(tw->notebook), tt->hbox, TRUE);
+    gtk_notebook_set_tab_reorderable (GTK_NOTEBOOK(tw->notebook), tt->hbox, TRUE);
 
-    /* We should show the tabs if there are more than one tab in the notebook */
-    if (gtk_notebook_get_n_pages (GTK_NOTEBOOK (tw->notebook)) > 1)
+    /* We should show the tabs if there are more than one tab in the notebook,
+     * and tab position is not set to hidden */
+    if (gtk_notebook_get_n_pages (GTK_NOTEBOOK (tw->notebook)) > 1 &&
+            config_getint("tab_pos") != NB_HIDDEN)
         gtk_notebook_set_show_tabs (GTK_NOTEBOOK (tw->notebook), TRUE);
 
     /* Add to GList list of tilda_term structures in tilda_window structure */
@@ -711,14 +718,6 @@ gint tilda_window_add_tab (tilda_window *tw)
     return TRUE; //index;
 }
 
-/**
- * tilda_window_close_tab ()
- *
- * Closes the tab at the given tab index (starting from 0)
- *
- * Success: return 0
- * Failure: return non-zero
- */
 gint tilda_window_close_tab (tilda_window *tw, gint tab_index, gboolean force_exit)
 {
     DEBUG_FUNCTION ("tilda_window_close_tab");
@@ -772,7 +771,7 @@ gint tilda_window_close_tab (tilda_window *tw, gint tab_index, gboolean force_ex
                     break;
                 case RESTART_TERMINAL_AND_HIDE:
                     tilda_window_add_tab (tw);
-                    pull (tw, PULL_UP);
+                    pull (tw, PULL_UP, TRUE);
                     break;
                 case EXIT_TILDA:
                 default:
